technique T0
{
    pass P0
    {
		SpecularEnable = true;

        AddressU[0]		= Wrap;
		AddressV[0]		= Wrap;
		MinFilter[0]	= Linear;
		MagFilter[0]	= Linear;
		MipFilter[0]	= Point;

        AddressU[1]		= Wrap;
		AddressV[1]		= Wrap;
		MinFilter[1]	= Linear;
		MagFilter[1]	= Linear;
		MipFilter[1]	= Linear;

        AddressU[2]		= Clamp;
		AddressV[2]		= Clamp;
		MinFilter[2]	= Linear;
		MagFilter[2]	= Linear;
		MipFilter[2]	= Point;

        VertexShader =
        decl
        {
            stream 0;
            float v0[3];	// Position
            stream 1;
            short v1[2];	// Texture coord1
            stream 2;
            D3DCOLOR v2;	// Normal
            stream 3;
            D3DCOLOR v3;	// Basis, S
            D3DCOLOR v4;	// Basis, T
            stream 4;
            D3DCOLOR v6;	// Influences indices
            stream 6;
            D3DCOLOR v5;	// Colors
        }
		asm
		{
			vs.1.1

			mul		r0, v6, c91.y

			// First Influence on position
			mov		a0.x, r0.x
			m4x3	r1, v0, c[a0.x]
			sge		r1.w, r0, r0

			// Perform blend
			mad		r3, v0, -c91.w, v0
			mad		r3, r1, c91.w, r3

			// Apply view+projection to result
			m4x4	r1, r3, c92
			mov		oPos, r1
			mov		oFog.x, r1.w					// Handle fog

			mov		a0.x, r0.x
			mad		r4, v3, c91.z, -v3.w			// EXPECTING 1.0 to be in v2.w
			m3x3	r0, r4, c[a0.x]					// Deform S
			mad		r4, v4, c91.z, -v4.w			// EXPECTING 1.0 to be in v2.w
			m3x3	r1, r4, c[a0.x]					// Deform T
			mad		r4, v2, c91.z, -v2.w			// EXPECTING 1.0 to be in v2.w
			m3x3	r2, r4, c[a0.x]					// Deform SxT

			// Transform light0 direction vector into tangent space
			dp3		r5.x,	r0,			-c88					// xform light into tangent space
			dp3		r5.y,	r1,			-c88
			dp3		r5.z,	r2,			-c88

			dp3		r5.w,	r5,			r5						// Normalize
			rsq		r5.w,	r5.w
			mul		r5,		r5,			r5.w

			mad		oD0.xyz,	r5,		c86.y,		c86.y		// Transform from -1..1 to 0..1, half == 0.5f
			mov		oD0.w,		v5.y							// Use the green component of the color


			// transform vertex position to world space
			//  to calculate V, vector to viewer in world
			//  space.

			// Half angle vector is  (L+V)/||L+V||  or Normalize( L+V )
			//  ||a|| is magnitude of a
			//  L = vec to light from vertex point
			//  V = vec to viewer from vertex point
			// vertex position - eye position
			// eye position - vertex position
			add		r5,  c90.xyz, -r3.xyz

			// Normalize eye vec
			dp3		r5.w, r5, r5
			rsq		r5.w, r5.w
			mul		r5, r5, r5.w

			// Add them to average & create half angle vector
			add		r4,  -c88, r5

			//  Normalize it
			dp3		r4.w, r4, r4
			rsq		r4.w, r4.w
			mul		r4,   r4, r4.w

			/////////////////////////////////////////////////
			// Transform half angle vector to local space and
			//  output to specular register for specular calc

			dp3		r6.x, r4, r0
			dp3		r6.y, r4, r1
			dp3		r6.z, r4, r2
			mad		oD1.xyz,	r6,		c86.y,		c86.y		// Transform from -1..1 to 0..1, half == 0.5f


			// Move the uvs through... (they are scaled up, so we have to bring 'em down)
			mul		oT0.xyz, v1, c87
			mul		oT1.xyz, v1, c87

			// Compute uvs for shadow buffers
			m4x4	r1, r3, c72
			max		oT2, r1, c86.wwwz	// Clamps W to be >= 0.0 (xyz should pass through)			
		};

        PixelShader = 
        asm
        {
            xps.1.1

			// c0 = ambient color
			// c1 = diffuse color
			// c2 = specular color
			// c3 = backlight color

			tex t0													// Normal map
			tex t1													// Detail texture map
			tex t2													// Shadow map

			xdd r0.rgb, r1.rgb, t0_bx2, v0_bx2, t0_bx2, v1_bx2		// r0.rgb = diffuse intensity (t0_bx2 dot v0_bx2)
			+ mul t2.a, t2.b, v0.a									// r1.rgb = specular intensity (t0_bx2 dot v1_bx2)
																	// t2.a = shadow intensity (t2.a = t2.b * (v0.a))
																	// We store the complement of the ambient occlusion in
																	// the alpha channel. We do this to simply support
																	// the case where there is no channel of ambient occlusion
																	// supplied. In this case, the values will be zero
																	// (1-zero = 1).

			xmma t0.rgb, r1.rgb, discard.rgb, r0_sat, t2.a, r0, -c3	// r0.rgb = shadowed diffuse intensity (r0_sat * t2.a)
			+ mul r1.a, r1_sat.b, r1_sat.b							// r1.a = specular intensity ^ 2
			
			mul_d2 r0.rgb, r0_sat, c0								// r0.rgb = partial ambient
			+ mul r1.a, r1.a, r1.a									// r1.a = specular intensity ^ 4

			mad r1.rgb, r0, 1-t2.a, r1_sat							// r1.rgb = partial ambient * (1 - shadow) + backlighting
			+ mul r1.a, r1.a, r1.a									// r1.a = specular intensity ^ 8

			mad r0.rgb, t0, c1, c0									// r0.rgb = final diffuse
			+ mul r1.a, r1.a, t0.a									// r1.a = specular intensity * specular map

			add r0.rgb, r0, r1										// r0.rgb = final diffuse + final backlighting
			+ mul t1.a, t1.a, c1.a

			mul t2.rgb, t2.a, c2									// t2.rgb = specular color * shadow

			mul r0, r0, t1
			
			xfc fog.a, r0, fog, prod, r1.a, t2, t1.a				// final color
//xfc t3, 1-zero, zero, zero, zero, zero, t1.a					// final color
        };
    }
}
