technique T0
{
    pass P0
    {
		// Disable specular
		SpecularEnable = False;
		
		// Set Z-buffer
		ZEnable			= True;
		ZWriteEnable	= True;
				
		// Set alpha blending mode
		AlphaBlendEnable = False;
		
		// Set texture address mode and min mag filters
		AddressU[0]		= Wrap;
		AddressV[0]		= Wrap;
		MipFilter[0]	= Linear;
		MinFilter[0]	= Linear;
		MagFilter[0]	= Linear;

		// Set shadow buffer args
        AddressU[1]		= Clamp;
		AddressV[1]		= Clamp;
		MipFilter[1]	= Linear;
		MinFilter[1]	= Linear;
		MagFilter[1]	= Linear;
			
        VertexShader =
        decl
        {
            stream 0;
            float v0[3];	// Position
            D3DCOLOR v3;	// Normal
            stream 1;
            short v7[2];	// tex coord1
        }
		asm
		{
			vs.1.1

			// Apply view+projection to result
			m4x4	r0, v0, c92
			mov		oPos, r0
			mov		oFog.x, r0.w
			
			// Lighting (diffuse + ambient + emissive)
			mad		r0, v3, c91.z, -v3.w	// EXPECTING 1.0 to be in v3.w
			dp3		r2, r0, -c81			// Dot for diffuse
			lit		r2, r2
			mul		oD0, r2.yyyw, c83		// Diffuse

			// Move the uvs through...
			mul		r0, v7, c91.x
			add		oT0.xyz, r0, c60

			// Compute uvs for shadow buffers
			m4x4	r1, v0, c76
			max		oT1, r1, c86.wwwz		// Clamps W to be >= 0.0 (xyz should pass through)
		};

		PixelShader = 
        asm
        {
			// Declare pixel shader version 1.1
			ps.1.1

			// Define textures
			tex t0				// water texture (currently unused)
			tex t1				// lightmap

//			add t0, t0, c1
			mad r0.rgb, v0, t1, c0	// calculate shadow and add ambient
			+mov r0.a, v0.a
			mul r0, r0, c1
        };   
    }
    
    pass P1
    {
        // Z-buffering to be used
        ZWriteEnable = False;
        ZEnable = True;
 
		// Disable alpha test
		AlphaTestEnable = False;
		
		// Set Alpha blend settings
		AlphaBlendEnable = True;
		SrcBlend = SrcAlpha;
		DestBlend = InvSrcAlpha;
		
        AddressU[0]		= Wrap;
		AddressV[0]		= Wrap;
		MipFilter[0]	= Linear;
		MinFilter[0]	= Linear;
		MagFilter[0]	= Linear;

        AddressU[1]		= Clamp;
		AddressV[1]		= Clamp;
		MipFilter[1]	= Linear;
		MinFilter[1]	= Linear;
		MagFilter[1]	= Linear;

        AddressU[2]		= Clamp;
		AddressV[2]		= Clamp;
		MipFilter[2]	= Linear;
		MinFilter[2]	= Linear;
		MagFilter[2]	= Linear;
		
		AddressU[3]		= Clamp;
		AddressV[3]		= Clamp;
		MipFilter[3]	= Linear;
		MinFilter[3]	= Linear;
		MagFilter[3]	= Linear;

        VertexShader =
        decl
        {
            stream 0;
            float v0[3];	// Position
            D3DCOLOR v3;	// Normal
            stream 1;
            short v7[2];	// tex coord1
        }
		asm
		{
			vs.1.1

			// Apply view+projection to result
			m4x4	r0, v0, c92
			mov		oPos, r0
			mov		oFog.x, r0.w

			// Calculate the eye vector in world space
			add r1, -v0, c90
			m3x3 r0, r1, c4								// Now transform the eye vector by the reflection matrix

			mad		oD0.xyz,	r0,		c1,		c1		// Transform from -1..1 to 0..1

			mul	oT0, v0, c0

			mov oT1, c4									// Move the reflection rotation matrix into the uvs
			mov oT2, c5
			mov oT3, c6

			mov	oT1.w, r0.x
			mov oT2.w, r0.y
			mov oT3.w, r0.z
		};

        PixelShader = 
        asm
        {
			// Declare pixel shader version 1.1
			ps.1.1

			// Define t0 as a standard 3-vector from bumpmap
			tex t0

			// Perform matrix multiply to get a local normal bump.  Then
			// reflect the eye vector through the normal and sample from 
			// a cubic environment map.
			// Must use _bx2 to convert unsigned texture normal map values
			//  to the full [-1,1] signed range

			texm3x3pad		t1,	t0_bx2
			texm3x3pad		t2,	t0_bx2
			texm3x3vspec	t3,	t0_bx2

			dp3_x2 r0, t0_bx2, v0_bx2		// dot the normal with the eye vector (from surface to eye)

			// now compute (1-cos(x))^8 - a reasonable approx. to fresnel blending factor (cos(x) is result of prev. dot product)
			mul r0, r0, r0
			mul r0, r0, r0
			mov r0, 1-r0

			sub r1, c1, c0					// fresnel max (c1) - fresnel min (c0)
			mad r0, r0, r1, c0

			mov r0.a, r0.b			
			mov r0.rgb, t3					// move the actual texture into rgb
        };
    }    
}
